@@ -40,16 +40,16 @@ __docformat__ = "restructuredtext en"
 # ------------------------------------------------------------------------------
 
 from pelix.ipopo.decorators import ComponentFactory, Provides, Validate, \
-    Invalidate, Property, Requires, Bind, Unbind
+    Invalidate, Property, Requires, BindField, UnbindField, UpdateField
 
 import pelix.ipopo.constants as constants
-import pelix.framework as pelix
 import pelix.utilities as utilities
 
 import logging
 import socket
 import sys
 import threading
+import traceback
 
 # Basic HTTP server
 if sys.version_info[0] == 3:
@@ -238,7 +238,13 @@ class _RequestHandler(BaseHTTPRequestHandler):
 
                 # Create a wrapper to pass the handler to the servlet
                 def wrapper():
-                    return getattr(servlet, name)(request, response)
+                    try:
+                        # Handle the request
+                        return getattr(servlet, name)(request, response)
+
+                    except:
+                        # Send a 500 error page on error
+                        return self.send_exception(response)
 
                 # Return it
                 return wrapper
@@ -279,6 +285,34 @@ class _RequestHandler(BaseHTTPRequestHandler):
         response = _HTTPServletResponse(self)
         response.send_content(404, page)
 
+
+    def send_exception(self, response):
+        """
+        Sends an exception page with a 500 error code.
+        Must be called from inside the exception handling block.
+        
+        :param response: The response handler
+        """
+        # Get a formatted stack trace
+        stack = traceback.format_exc()
+
+        # Prepare the page content
+        page = """<html>
+<head>
+<title>500 - Internal Server Error</title>
+</head>
+<body>
+<h1>Internal Server Error</h1>
+<p>Error hanling request upon: {0}</p>
+<pre>
+{1}
+</pre>
+</body>
+</html>""".format(self.path, stack)
+
+        # Send the page
+        response.send_content(500, page)
+
 # ------------------------------------------------------------------------------
 
 class _HttpServerFamily(ThreadingMixIn, HTTPServer):
@@ -355,6 +389,9 @@ class HttpService(object):
         self._logger_name = None
         self._logger_level = None
 
+        # Validation flag
+        self._validated = False
+
         # The logger
         self._logger = None
 
@@ -367,6 +404,11 @@ class HttpService(object):
         # Field injected by iPOPO
         self._servlets_services = None
 
+        # Servlet -> ServiceReference
+        self._servlets_refs = {}
+        self._binding_lock = threading.Lock()
+
+        # Server control
         self._server = None
         self._thread = None
 
@@ -413,35 +455,71 @@ class HttpService(object):
         return False
 
 
-    @Bind
-    def _bind(self, service, service_reference):
+    def __register_servlet_service(self, service, service_reference):
+        """
+        Registers a servlet according to its service properties
+        
+        :param service: A servlet service
+        :param service_reference: The associated ServiceReference
+        """
+        # Servlet bound
+        paths = service_reference.get_property(http.HTTP_SERVLET_PATH)
+        if utilities.is_string(paths):
+            # Register the servlet to a single path
+            self.register_servlet(paths, service, None)
+
+        elif isinstance(paths, (list, tuple)):
+            # Register the servlet to multiple paths
+            for path in paths:
+                self.register_servlet(path, service, None)
+
+
+    @BindField("_servlets_services")
+    def _bind(self, field, service, service_reference):
         """
         Called by iPOPO when a service is bound
         """
-        specifications = service_reference.get_property(pelix.OBJECTCLASS)
-        if http.HTTP_SERVLET in specifications:
-            # Servlet bound
-            paths = service_reference.get_property(http.HTTP_SERVLET_PATH)
-            if utilities.is_string(paths):
-                # Register the servlet to a single path
-                self.register_servlet(paths, service, None)
+        with self._binding_lock:
+            self._servlets_refs[service] = service_reference
 
-            elif isinstance(paths, (list, tuple)):
-                # Register the servlet to multiple paths
-                for path in paths:
-                    self.register_servlet(path, service, None)
+            if self._validated:
+                # We've been validated, register the service
+                self.__register_servlet_service(service, service_reference)
 
 
-    @Unbind
-    def _unbind(self, service, service_reference):
+    @UpdateField('_servlets_services')
+    def _update(self, field, service, service_reference, old_properties):
+        """
+        Called by iPOPO when the properties of a service have been updated
+        """
+        # Check if the property concerns the registration
+        old_path = old_properties.get(http.HTTP_SERVLET_PATH)
+        new_path = service_reference.get_property(http.HTTP_SERVLET_PATH)
+        if old_path == new_path:
+            # Nothing to do
+            return
+
+        with self._binding_lock:
+            # Unregister the previous paths
+            self.unregister(None, service)
+
+            if self._validated:
+                # Register the service with its new properties
+                self.__register_servlet_service(service, service_reference)
+
+
+    @UnbindField("_servlets_services")
+    def _unbind(self, field, service, service_reference):
         """
         Called by iPOPO when a service is gone
         """
-        specifications = service_reference.get_property(pelix.OBJECTCLASS)
-        if http.HTTP_SERVLET in specifications:
+        with self._binding_lock:
             # Servlet gone: unregister all paths associated to this servlet
             self.unregister(None, service)
 
+            # Remove the service reference
+            del self._servlets_refs[service]
+
 
     def get_access(self):
         """
@@ -477,10 +555,20 @@ class HttpService(object):
         # Use lower case for comparison
         path = path.lower()
 
+        if path[-1] != '/':
+            # Add a trailing slash
+            path += '/'
+
         with self._lock:
             longest_match = ""
             for servlet_path in self._servlets.keys():
-                if path.startswith(servlet_path):
+
+                tested_path = servlet_path
+                if tested_path[-1] != '/':
+                    # Add a trailing slash
+                    tested_path += '/'
+
+                if path.startswith(tested_path):
                     # Found a corresponding servlet
                     if len(servlet_path) > len(longest_match):
                         # And its deeper than the previous one
@@ -521,9 +609,6 @@ class HttpService(object):
         if parameters is None:
             parameters = {}
 
-        parameters['http.address'] = self._address
-        parameters['http.port'] = self._port
-
         with self._lock:
             if path in self._servlets:
                 # Already registered path
@@ -535,6 +620,10 @@ class HttpService(object):
                     raise ValueError("A servlet is already registered on {0}" \
                                      .format(path))
 
+            # Add server information in parameters
+            parameters['http.address'] = self._address
+            parameters['http.port'] = self._port
+
             # Call back the method
             if self.__safe_callback(servlet, "bound_to", path, parameters):
                 # Store the servlet
@@ -660,6 +749,14 @@ class HttpService(object):
         self._thread.daemon = True
         self._thread.start()
 
+        with self._binding_lock:
+            # Set the validation flag up, once the server is ready
+            self._validated = True
+
+            # Register bound servlets
+            for service, svc_ref in self._servlets_refs.items():
+                self.__register_servlet_service(service, svc_ref)
+
         self.log(logging.INFO, "HTTP server started: [%s]:%d ...",
                  self._address, self._port)
 
@@ -669,6 +766,14 @@ class HttpService(object):
         """
         Component invalidation
         """
+        with self._binding_lock:
+            # Refuse new registrations
+            self._validated = False
+
+            # Unregister servlets (to call unbound_from...)
+            for service in self._servlets_refs:
+                self.unregister(None, service)
+
         self.log(logging.INFO, "Shutting down HTTP server: [%s]:%d ...",
                  self._address, self._port)
 
