@@ -4,9 +4,9 @@
 Defines the iPOPO decorators classes to manipulate component factory classes
 
 :author: Thomas Calmant
-:copyright: Copyright 2012, isandlaTech
+:copyright: Copyright 2013, isandlaTech
 :license: GPLv3
-:version: 0.4
+:version: 0.5
 :status: Alpha
 
 ..
@@ -27,20 +27,21 @@ Defines the iPOPO decorators classes to manipulate component factory classes
     along with iPOPO. If not, see <http://www.gnu.org/licenses/>.
 """
 
-__version__ = (0, 4, 0)
+# Module version
+__version_info__ = (0, 5, 0)
+__version__ = ".".join(map(str, __version_info__))
 
 # Documentation strings format
 __docformat__ = "restructuredtext en"
 
 # ------------------------------------------------------------------------------
 
+# Pelix modules
 from pelix.utilities import is_string
 from pelix.ipopo.core import FactoryContext, Requirement
-
 import pelix.ipopo.constants as constants
 
-# ------------------------------------------------------------------------------
-
+# Standard library
 import inspect
 import logging
 import threading
@@ -229,6 +230,59 @@ def _ipopo_setup_callback(cls, context):
     context.callbacks.clear()
     context.callbacks.update(callbacks)
 
+
+def _ipopo_setup_field_callback(cls, context):
+    """
+    Sets up the class _field_callback dictionary
+    
+    :param cls: The class to handle
+    :param context: The factory class context
+    """
+    assert inspect.isclass(cls)
+    assert isinstance(context, FactoryContext)
+
+    if context.field_callbacks is not None:
+        callbacks = context.field_callbacks.copy()
+
+    else:
+        callbacks = {}
+
+    functions = inspect.getmembers(cls, inspect.isroutine)
+
+    for name, function in functions:
+        if not hasattr(function, constants.IPOPO_METHOD_FIELD_CALLBACKS):
+            # No attribute, get the next member
+            continue
+
+        method_callbacks = getattr(function,
+                                   constants.IPOPO_METHOD_FIELD_CALLBACKS)
+        if not isinstance(method_callbacks, list):
+            # Invalid content
+            _logger.warning("Invalid attribute %s in %s", \
+                            constants.IPOPO_METHOD_FIELD_CALLBACKS, name)
+            continue
+
+        # Keeping it allows inheritance : by removing it, only the first
+        # child will see the attribute -> Don't remove it
+
+        # Store the call backs
+        for kind, field in method_callbacks:
+            fields_cbs = callbacks.setdefault(field, {})
+
+            if kind in fields_cbs and \
+            not is_from_parent(cls, fields_cbs[kind].__name__):
+                _logger.warning("Redefining the callback %s in '%s'. " \
+                                "Previous callback : '%s' (%s). " \
+                                "New callback : %s", kind, name,
+                                fields_cbs[kind].__name__,
+                                fields_cbs[kind], function)
+
+            fields_cbs[kind] = function
+
+    # Update the factory context
+    context.field_callbacks.clear()
+    context.field_callbacks.update(callbacks)
+
 # ------------------------------------------------------------------------------
 
 def _append_object_entry(obj, list_name, entry):
@@ -382,8 +436,11 @@ class ComponentFactory:
     Decorator that sets up a component factory class
     """
 
-    # Non inheritable fields, to clean up during manipulation if needed
     NON_INHERITABLE_FIELDS = (constants.IPOPO_INSTANCES,)
+    """
+    Non inheritable fields, i.e. to remove during the manipulation of child
+    classes, if needed
+    """
 
     def __init__(self, name=None):
         """
@@ -417,6 +474,7 @@ class ComponentFactory:
 
         # Find callbacks
         _ipopo_setup_callback(factory_class, context)
+        _ipopo_setup_field_callback(factory_class, context)
 
         # Clean up inherited fields, to avoid weird behavior
         for field in ComponentFactory.NON_INHERITABLE_FIELDS:
@@ -686,6 +744,165 @@ class Requires:
 
 # ------------------------------------------------------------------------------
 
+class BindField(object):
+    """
+    BindField callback decorator, called when a component is bound to a
+    dependency, injected in the given field.
+    
+    The decorated method must have the following prototype :
+    
+    .. python::
+       def bind_method(self, field, service, service_reference):
+           '''
+           Method called when a service is bound to the component
+           
+           :param field: Field wherein the dependency is injected
+           :param service: The injected service instance.
+           :param service_reference: The injected service ServiceReference
+           '''
+           # ...
+    
+    If the service is a required one, the bind callback is called **before** the
+    component is validated.
+    The bind field callback is called **after** the global bind method.
+    
+    The service reference can be stored *if its reference is deleted on unbind*.
+    
+    Exceptions raised by a bind callback are ignored.
+    """
+    def __init__(self, field):
+        """
+        Sets up the decorator
+        
+        :param field: Field associated to the binding
+        """
+        self._field = field
+
+
+    def __call__(self, method):
+        """
+        Updates the "field callback" list for this method
+        
+        :param method: Method to decorate
+        :return: Decorated method
+        :raise TypeError: The decorated element is not a valid function
+        """
+        if not inspect.isroutine(method):
+            raise TypeError("@BindField can only be applied on functions")
+
+        # Tests the number of parameters
+        validate_method_arity(method, "field", "service", "service_reference")
+
+        _append_object_entry(method, constants.IPOPO_METHOD_FIELD_CALLBACKS,
+                             (constants.IPOPO_CALLBACK_BIND_FIELD,
+                              self._field))
+        return method
+
+
+class UpdateField(object):
+    """
+    UpdateField callback decorator, called when a component dependency property
+    has been modified.
+    
+    The decorated method must have the following prototype :
+    
+    .. python::
+       def update_method(self, service, service_reference, old_properties):
+           '''
+           Method called when a service is bound to the component
+           
+           :param service: The injected service instance.
+           :param service_reference: The injected service ServiceReference
+           :param old_properties: Previous service properties
+           '''
+           # ...
+    
+    Exceptions raised by an update callback are ignored.
+    """
+    def __init__(self, field):
+        """
+        Sets up the decorator
+        
+        :param field: Field associated to the binding
+        """
+        self._field = field
+
+
+    def __call__(self, method):
+        """
+        Updates the "field callback" list for this method
+        
+        :param method: Method to decorate
+        :return: Decorated method
+        :raise TypeError: The decorated element is not a valid function
+        """
+        if not inspect.isroutine(method):
+            raise TypeError("@UnbindField can only be applied on functions")
+
+        # Tests the number of parameters
+        validate_method_arity(method, "field", "service", "service_reference",
+                              "old_properties")
+
+        _append_object_entry(method, constants.IPOPO_METHOD_FIELD_CALLBACKS,
+                             (constants.IPOPO_CALLBACK_UPDATE_FIELD,
+                              self._field))
+        return method
+
+
+class UnbindField(object):
+    """
+    UnbindField callback decorator, called when a component is unbound to a
+    dependency, removed from the given field.
+    
+    The decorated method must have the following prototype :
+    
+    .. python::
+       def unbind_method(self, field, service, service_reference):
+           '''
+           Method called when a service is bound to the component
+           
+           :param field: Field wherein the dependency is injected
+           :param service: The injected service instance.
+           :param service_reference: The injected service ServiceReference
+           '''
+           # ...
+    
+    If the service is a required one, the unbind callback is called **after**
+    the component has been invalidated.
+    The unbind field callback is called **before** the global unbind method.
+    
+    Exceptions raised by an unbind callback are ignored.
+    """
+    def __init__(self, field):
+        """
+        Sets up the decorator
+        
+        :param field: Field associated to the binding
+        """
+        self._field = field
+
+
+    def __call__(self, method):
+        """
+        Updates the "field callback" list for this method
+        
+        :param method: Method to decorate
+        :return: Decorated method
+        :raise TypeError: The decorated element is not a valid function
+        """
+        if not inspect.isroutine(method):
+            raise TypeError("@UnbindField can only be applied on functions")
+
+        # Tests the number of parameters
+        validate_method_arity(method, "field", "service", "service_reference")
+
+        _append_object_entry(method, constants.IPOPO_METHOD_FIELD_CALLBACKS,
+                             (constants.IPOPO_CALLBACK_UNBIND_FIELD,
+                              self._field))
+        return method
+
+# ------------------------------------------------------------------------------
+
 def Bind(method):
     """
     Bind callback decorator, called when a component is bound to a dependency.
@@ -693,11 +910,11 @@ def Bind(method):
     The decorated method must have the following prototype :
     
     .. python::
-       def bind_method(self, injected_service, service_reference):
+       def bind_method(self, service, service_reference):
            '''
            Method called when a service is bound to the component
            
-           :param injected_service: The injected service instance.
+           :param service: The injected service instance.
            :param service_reference: The injected service ServiceReference
            '''
            # ...
@@ -723,6 +940,41 @@ def Bind(method):
     return method
 
 
+def Update(method):
+    """
+    Update callback decorator, called when a component dependency property has
+    been modified.
+    
+    The decorated method must have the following prototype :
+    
+    .. python::
+       def update_method(self, service, service_reference, old_properties):
+           '''
+           Method called when a service is bound to the component
+           
+           :param service: The injected service instance.
+           :param service_reference: The injected service ServiceReference
+           :param old_properties: Previous service properties
+           '''
+           # ...
+    
+    Exceptions raised by an update callback are ignored.
+    
+    :param method: The decorated method
+    :raise TypeError: The decorated element is not a valid function
+    """
+    if type(method) is not types.FunctionType:
+        raise TypeError("@Update can only be applied on functions")
+
+    # Tests the number of parameters
+    validate_method_arity(method, "service", "service_reference",
+                          "old_properties")
+
+    _append_object_entry(method, constants.IPOPO_METHOD_CALLBACKS, \
+                         constants.IPOPO_CALLBACK_UPDATE)
+    return method
+
+
 def Unbind(method):
     """
     Unbind callback decorator, called when a component dependency is unbound.
@@ -730,11 +982,12 @@ def Unbind(method):
     The decorated method must have the following prototype :
     
     .. python::
-       def unbind_method(self, injected_instance):
+       def unbind_method(self, service, service_reference):
            '''
            Method called when a service is bound to the component
            
-           :param injected_instance: The injected service instance.
+           :param service: The injected service instance.
+           :param service_reference: The injected service ServiceReference
            '''
            # ...
     
